@@ -427,17 +427,11 @@ def _set_url_database(url: sa.engine.url.URL, database):
     :param database: New database to set.
 
     """
-    if hasattr(sa.engine, 'URL'):
-        ret = sa.engine.URL.create(
-            drivername=url.drivername,
-            username=url.username,
-            password=url.password,
-            host=url.host,
-            port=url.port,
-            database=database,
-            query=url.query
-        )
+    if hasattr(url, '_replace'):
+        # Cannot use URL.set() as database may need to be set to None.
+        ret = url._replace(database=database)
     else:  # SQLAlchemy <1.4
+        url = copy(url)
         url.database = database
         ret = url
     assert ret.database == database, ret
@@ -445,7 +439,7 @@ def _set_url_database(url: sa.engine.url.URL, database):
 
 
 def _get_scalar_result(engine, sql):
-    with engine.connect() as conn:
+    with engine.begin() as conn:
         return conn.scalar(sql)
 
 
@@ -480,7 +474,7 @@ def database_exists(url):
 
     """
 
-    url = copy(make_url(url))
+    url = make_url(url)
     database = url.database
     dialect_name = url.get_dialect().name
     engine = None
@@ -491,7 +485,7 @@ def database_exists(url):
                 url = _set_url_database(url, database=db)
                 engine = sa.create_engine(url)
                 try:
-                    return bool(_get_scalar_result(engine, text))
+                    return bool(_get_scalar_result(engine, sa.text(text)))
                 except (ProgrammingError, OperationalError):
                     pass
             return False
@@ -501,7 +495,7 @@ def database_exists(url):
             engine = sa.create_engine(url)
             text = ("SELECT SCHEMA_NAME FROM INFORMATION_SCHEMA.SCHEMATA "
                     "WHERE SCHEMA_NAME = '%s'" % database)
-            return bool(_get_scalar_result(engine, text))
+            return bool(_get_scalar_result(engine, sa.text(text)))
 
         elif dialect_name == 'sqlite':
             url = _set_url_database(url, database=None)
@@ -509,14 +503,14 @@ def database_exists(url):
             if database:
                 return database == ':memory:' or _sqlite_file_exists(database)
             else:
-                # The default SQLAlchemy database is in memory, and :memory is
+                # The default SQLAlchemy database is in memory, and :memory: is
                 # not required, thus we should support that use case.
                 return True
         else:
             text = 'SELECT 1'
             try:
                 engine = sa.create_engine(url)
-                return bool(_get_scalar_result(engine, text))
+                return bool(_get_scalar_result(engine, sa.text(text)))
             except (ProgrammingError, OperationalError):
                 return False
     finally:
@@ -546,7 +540,7 @@ def create_database(url, encoding='utf8', template=None):
     other database engines should be supported.
     """
 
-    url = copy(make_url(url))
+    url = make_url(url)
     database = url.database
     dialect_name = url.get_dialect().name
     dialect_driver = url.get_dialect().driver
@@ -555,6 +549,8 @@ def create_database(url, encoding='utf8', template=None):
         url = _set_url_database(url, database="postgres")
     elif dialect_name == 'mssql':
         url = _set_url_database(url, database="master")
+    elif dialect_name == 'cockroachdb':
+        url = _set_url_database(url, database="defaultdb")
     elif not dialect_name == 'sqlite':
         url = _set_url_database(url, database=None)
 
@@ -575,27 +571,27 @@ def create_database(url, encoding='utf8', template=None):
             quote(engine, template)
         )
 
-        with engine.connect() as connection:
-            connection.execute(text)
+        with engine.begin() as connection:
+            connection.execute(sa.text(text))
 
     elif dialect_name == 'mysql':
         text = "CREATE DATABASE {0} CHARACTER SET = '{1}'".format(
             quote(engine, database),
             encoding
         )
-        with engine.connect() as connection:
-            connection.execute(text)
+        with engine.begin() as connection:
+            connection.execute(sa.text(text))
 
     elif dialect_name == 'sqlite' and database != ':memory:':
         if database:
-            with engine.connect() as connection:
-                connection.execute("CREATE TABLE DB(id int);")
-                connection.execute("DROP TABLE DB;")
+            with engine.begin() as connection:
+                connection.execute(sa.text("CREATE TABLE DB(id int);"))
+                connection.execute(sa.text("DROP TABLE DB;"))
 
     else:
         text = 'CREATE DATABASE {0}'.format(quote(engine, database))
-        with engine.connect() as connection:
-            connection.execute(text)
+        with engine.begin() as connection:
+            connection.execute(sa.text(text))
 
     engine.dispose()
 
@@ -613,7 +609,7 @@ def drop_database(url):
 
     """
 
-    url = copy(make_url(url))
+    url = make_url(url)
     database = url.database
     dialect_name = url.get_dialect().name
     dialect_driver = url.get_dialect().driver
@@ -622,6 +618,8 @@ def drop_database(url):
         url = _set_url_database(url, database="postgres")
     elif dialect_name == 'mssql':
         url = _set_url_database(url, database="master")
+    elif dialect_name == 'cockroachdb':
+        url = _set_url_database(url, database="defaultdb")
     elif not dialect_name == 'sqlite':
         url = _set_url_database(url, database=None)
 
@@ -637,7 +635,7 @@ def drop_database(url):
         if database:
             os.remove(database)
     elif dialect_name == 'postgresql':
-        with engine.connect() as connection:
+        with engine.begin() as connection:
             # Disconnect all users from the database we are dropping.
             version = connection.dialect.server_version_info
             pid_column = (
@@ -649,14 +647,14 @@ def drop_database(url):
             WHERE pg_stat_activity.datname = '%(database)s'
             AND %(pid_column)s <> pg_backend_pid();
             ''' % {'pid_column': pid_column, 'database': database}
-            connection.execute(text)
+            connection.execute(sa.text(text))
 
             # Drop the database.
             text = 'DROP DATABASE {0}'.format(quote(connection, database))
-            connection.execute(text)
+            connection.execute(sa.text(text))
     else:
         text = 'DROP DATABASE {0}'.format(quote(engine, database))
-        with engine.connect() as connection:
-            connection.execute(text)
+        with engine.begin() as connection:
+            connection.execute(sa.text(text))
 
     engine.dispose()
