@@ -16,7 +16,7 @@ class CreateView(DDLElement):
 def compile_create_materialized_view(element, compiler, **kw):
     return 'CREATE {}VIEW {} AS {}'.format(
         'MATERIALIZED ' if element.materialized else '',
-        element.name,
+        compiler.dialect.identifier_preparer.quote(element.name),
         compiler.sql_compiler.process(element.selectable, literal_binds=True),
     )
 
@@ -32,7 +32,7 @@ class DropView(DDLElement):
 def compile_drop_materialized_view(element, compiler, **kw):
     return 'DROP {}VIEW IF EXISTS {} {}'.format(
         'MATERIALIZED ' if element.materialized else '',
-        element.name,
+        compiler.dialect.identifier_preparer.quote(element.name),
         'CASCADE' if element.cascade else ''
     )
 
@@ -42,7 +42,8 @@ def create_table_from_selectable(
     selectable,
     indexes=None,
     metadata=None,
-    aliases=None
+    aliases=None,
+    **kwargs
 ):
     if indexes is None:
         indexes = []
@@ -59,7 +60,7 @@ def create_table_from_selectable(
         )
         for c in get_columns(selectable)
     ] + indexes
-    table = sa.Table(name, metadata, *args)
+    table = sa.Table(name, metadata, *args, **kwargs)
 
     if not any([c.primary_key for c in get_columns(selectable)]):
         table.append_constraint(
@@ -148,7 +149,9 @@ def create_view(
                 Column('premium_user', Boolean, default=False),
             )
 
-        premium_members = select([users]).where(users.c.premium_user == True)
+        premium_members = select(users).where(users.c.premium_user == True)
+        # sqlalchemy 1.3:
+        # premium_members = select([users]).where(users.c.premium_user == True)
         create_view('premium_users', premium_members, metadata)
 
         metadata.create_all(engine) # View is created at this point
@@ -188,8 +191,8 @@ def refresh_materialized_view(session, name, concurrently=False):
     # order to include newly-created/modified objects in the refresh.
     session.flush()
     session.execute(
-        'REFRESH MATERIALIZED VIEW {}{}'.format(
+        sa.text('REFRESH MATERIALIZED VIEW {}{}'.format(
             'CONCURRENTLY ' if concurrently else '',
-            name
-        )
+            session.bind.engine.dialect.identifier_preparer.quote(name)
+        ))
     )
